package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
 
/**
 * EJB Gen Delegate
 * @author <a href="mailto:ramesh.ramakrishnan@upco.co.uk">Ramesh Ramakrishnan</a>
 * @version 1.0.0
 * 
 * Description: EJB Delegate to archive EJB Beans along with deployment descriptor
 * 
 * <p>The EJB Delegate generates archive file of an EJB project by including its container specific deployment descriptors
 * along with the Bean classes. This archiving is achieved by using ejbjar ant task. 
 * The generated archive can then be deployed as an EJB module in an EJB Container</p>
 */
/* 
 * @ant-target doEjbGen
 */
class EJBGenDelegate extends AbstractPluginTargetDelegate {

	/*
	 * Corresponds to descriptordir attribute of ejbjar task. Maps to the base directory containing EJB deployment descriptor files 
	 * @ant-property-name ejb.descriptor.dir
	 */
	protected String descriptorDir
	/*
	 * Corresponds to cmpversion attribute of ejbjar task.CMP 2.0 version exists only for JBoss. 
	 * @ant-property-name ejb.cmp.version
	 */
	protected String cmpVersion
	/*
	 * Corresponds to destinationdir attribute of ejbjar task. Maps to base directory for the generated Jar files
	 * @ant-property-name ejb.destination.dir
	 */
	protected String destinationDir
	/*
	 * Corresponds to dependency attribute of ejbjar task. Controls the inclusiong of additional classes and interfaces to the archive.
	 * @ant-property-name ejb.dependency
	 */
	protected String dependency
	/*
	 * Corresponds to basejarname attribute of ejbjar task. base name that is used for the generated jar files. This attribute is used when naming attribute has the value basejarname
	 * @ant-property-name ejb.base.jar.name
	 */
	protected String baseJarName
	/*
	 * Corresponds to genericjarsuffix attribute of ejbjar task . String value appended to the base name of the jar file.
	 * @ant-property-name generic.jar.suffix
	 */
	protected String genericJarSuffix
	/*
	 * Corresponds to srcdir of ejbjar attribute. Maps to the base directory containing bean classes 
	 * @ant-property-name ejb.src.dir
	 */
	protected String srcDir
	/*
	 * Corresponds to naming attribute of ejbjar task. 
	 * This attribute takes 4 possible values. descriptor attribute value is default naming scheme. Name of the generated Jar file is derived from the deployment descriptor
	 * ejb-name attribute value uses ejb-name element from the deployment descriptor. directory attribute value will derive the name of the generated bean jar from the directory containing 
	 * the deployment descriptor. basejarname attribute value indicates that the name of bean would be explicitly provided
	 * @ant-property-name ejb.naming
	 */
	protected String naming
	/*
	 * Corresponds to baseNameTerminator of ejbjar attribute. String value used to substring out of a string from the name of each deployment descriptor. This is not to be used when using the attribute, basejarname.
	 * @ant-property-name ejb.base.name.terminator
	 */
	protected String baseNameTerminator
	/*
	 * Corresponds to flatdestdir of ejbjar attribute.
	 * @ant-property-name ejb.flat.dest.dir
	 */
	protected String flatDestDir
	/*
	 * Directory to include additional files using support nested element of ejbjar task
	 * @ant-property-name ejb.support.dir
	 */
	protected String supportDir
	/*
	 * Include pattern in the inclusion directory
	 * @ant-property-name ejb.support.includes
	 */
	protected String supportIncludes
	/*
	 * Exclude pattern in the inclusion directory
	 * @ant-property-name ejb.support.excludes
	 */
	protected String supportExcludes

	/**
	 * Setter method for member variable descriptorDir
	 * @param descriptorDir	
	 */	
	void setDescriptorDir( String descriptorDir ){
		this.descriptorDir = descriptorDir
	}
	/**
	 * Setter method for member variable cmpVersion
	 * @param cmpVersion	
	 */	
	void setCmpVersion( String cmpVersion ){
		this.cmpVersion = cmpVersion
	}
	/**
	 * Setter method for member variable destinationDir
	 * @param destinationDir	
	 */	
	void setDestinationDir( String destinationDir ){
		this.destinationDir = destinationDir
	}
	/**
	 * Setter method for member variable dependency
	 * @param dependency	
	 */	
	void setDependency( String dependency ){
		this.dependency = dependency
	}
	/**
	 * Setter method for member variable baseJarName
	 * @param baseJarName	
	 */	
	void setBaseJarName( String baseJarName ){
		this.baseJarName = baseJarName
	}
	/**
	 * Setter method for member variable genericJarSuffix
	 * @param genericJarSuffix	
	 */	
	void setGenericJarSuffix( String genericJarSuffix ){
		this.genericJarSuffix = genericJarSuffix
	}
	/**
	 * Setter method for member variable srcDir
	 * @param srcDir	
	 */	
	void setSrcDir( String srcDir ){
		this.srcDir = srcDir
	}
	/**
	 * Setter method for member variable naming
	 * @param naming	
	 */	
	void setNaming( String naming ){
		this.naming = naming
	}
	/**
	 * Setter method for member variable baseNameTerminator
	 * @param baseNameTerminator	
	 */	
	void setBaseNameTerminator( String baseNameTerminator ){
		this.baseNameTerminator = baseNameTerminator
	}
	/**
	 * Setter method for member variable flatDestDir
	 * @param flatDestDir	
	 */	
	void setFlatDestDir( String flatDestDir ){
		this.flatDestDir = flatDestDir
	}
	/**
	 * Setter method for member variable supportDir
	 * @param supportDir	
	 */	
	void setSupportDir( String supportDir ){
		this.supportDir = supportDir
	}
	/**
	 * Setter method for member variable supportIncludes
	 * @param supportIncludes	
	 */	
	void setSupportIncludes( String supportIncludes ){
		this.supportIncludes = supportIncludes
	}
	/**
	 * Setter method for member variable supportExcludes
	 * @param supportExcludes	
	 */	
	void setSupportExcludes( String supportExcludes ){
		this.supportExcludes = supportExcludes
	}

	 /**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	 EJBGenDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	}
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */	
	void doCheck(){
		assert buildContext.get( "project.properties" ) != null
		assert descriptorDir != null
		assert cmpVersion != null
		assert destinationDir != null
		assert dependency != null
		assert baseJarName != null
		assert genericJarSuffix != null
		assert srcDir != null
		assert naming != null
		assert baseNameTerminator != null
		assert flatDestDir != null
		assert supportDir != null
		assert supportIncludes != null
		assert supportExcludes != null
	}	
	
	/**
	 * Concrete implementation of the AbstractDelegate.execute() method
	 */
	void doExecute(){
		def ejbDest = destinationDir + "/" + baseJarName
		
		def ejbOptions = [ "descriptordir":descriptorDir,
							"srcdir":srcDir,
							"destdir":destinationDir,
							"dependency":dependency,
							"genericjarsuffix":genericJarSuffix,
							"flatdestdir":flatDestDir,
							"cmpversion":cmpVersion]
		
		/* Use the correct attributes based on mutually exclusive attributes */
		if(naming.equals("basejarname")) {
			ejbOptions.put( "basejarname", baseJarName )
		} else {
			ejbOptions.put( "basenameterminator", baseNameTerminator )
		}
		
		ant.ejbjar(	ejbOptions ) {
			ant.support(dir:supportDir) {
				ant.include(name:supportIncludes)
				ant.exclude(name:supportExcludes)
			}
		}
		
		ant.property( name : "global.artifact.location", value : ejbDest )	
		
	}
		
}